import com.apama.EngineException;
import com.apama.event.EventListenerAdapter;
import com.apama.event.Event;
import com.apama.event.parser.*;
import com.apama.services.event.*;
import com.apama.util.Logger;
/*
 * ESChannelSample.java
 *
 * This sample demonstrates adding an EventServiceChannel to the EventService
 * and sending and receiving and event. 
 * 
 * Requirement:
 *
 * The ChannelledEvent.mon must be injected to the correlator before running this 
 * example.  The ChannelledEvent.mon simply re-sends the ChannelledEvent() event by
 * concatenate the string  "(echoed)" to the end of the s member.
 * 
 * In order to check the sendEvent() actually works, bring up the engine_receive
 * to keep track of the event  ChannelledEvent("hi there (channelled)")   echoed back
 * by the monitorscript.
 *
 * $Copyright(c) 2005 - 2010, 2012 Progress Software Corporation (PSC). All rights reserved.$
 * $Copyright (c) 2013-2015, 2017 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 *
 */

public class ESChannelSample {
	private static final Logger logger = Logger.getLogger(ESChannelSample.class);

	volatile boolean received = false;

	/**
	 * Simply send the ChannelledEvent("hi there")
	 * event to the correlator and process the response.
	 */
	public void runSample(String host, int port) {
		try
		(
			// get a IEventService instance from the EventServiceFactory
			// IEventService.close() will automatically get called upon exiting the try-with-resources block
			IEventService eventService = EventServiceFactory.createEventService(host, port, "ESChannelSample");
		) {
			// Add the channel to those monitored by the event service
			IEventServiceChannel ourChannel = eventService.addChannel(
					new String[]{"eventService.sample.channel"}, null);
	
			// Create an EventType to describe the event we are expecting to receive
			EventType channelEventType = new EventType("ChannelledEvent",
				FieldTypes.STRING.newField("s")
				);
	
			// Add a listener that will be notified whenever an event of the
			// specified type is received on the channel
			ourChannel.addEventListener(new MyEventListener(), channelEventType);
	
			// Send an event to the correlator
			try {
				Event cEvent = new Event("ChannelledEvent(\"hi there\")");
				eventService.sendEvent(cEvent);
			} catch (EngineException e) {
				logger.error("Failed to send event: ", e);
			}
	
			/**
			 * Since this is a console application, we need to wait for the callback to acknowledge
			 * the event response on the channel.  Otherwise this application will just terminate
			 * before receiving the response.
			 */
			while (!received) {
				try {
					Thread.sleep(500);
				} catch (InterruptedException ie) {
					logger.info("Interrupted while waiting: ", ie);
				}
	
			}
		}
	}

	/**
	 * Inner class to implement our callback method that is notified when events
	 * are received.
	 */
	private class MyEventListener extends EventListenerAdapter {

		/**
		 * Callback method that is called when a new Event is received.
		 * 
		 * @param event
		 *            The received Event.
		 */
		public void handleEvent(Event event) {
			System.out.println("MyEventListener.handleEvent() called, event = "
					+ event);
			// acknowledge the event response
			received = true;
		}
	}

	public static void main(String[] args) {
		if ((args.length != 2) || (Integer.parseInt(args[1]) <= 0)) {
			// Bad command line given
			System.out.println("Usage: java ESChannelSample <host> <port>");
			return;
		}
		String host = args[0];
		int port = Integer.parseInt(args[1]);
		new ESChannelSample().runSample(host, port);
	}

}
